NmTimerUpdater subclass: #NmWorldMapLocationBroadcaster
	instanceVariableNames: 'broadcastClient'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NetMorph-Server-MapServer'!

!NmWorldMapLocationBroadcaster methodsFor: 'actions' stamp: 'mu 6/2/2005 02:20'!
update
	NmServerManager isMapServerRunning ifFalse: [^self].
	self broadcastingPorts
		do: [:port | 
			self broadcastClient bindForBroadcast: port.
			self broadcastClient sendData: self locationMessage.
			(Delay forSeconds: self broadcastingIntervalSeconds) wait]! !


!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 5/29/2005 02:48'!
broadcastingIntervalSeconds
	^(self intervalSeconds // self broadcastingPorts size) max: 1! !

!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 6/3/2005 03:23'!
broadcastingPorts
	^NmServerSettings broadcastingWarpPorts! !

!NmWorldMapLocationBroadcaster methodsFor: 'private' stamp: 'mu 5/29/2005 03:52'!
locationMessage
	^NmBaseUtil localAddress, ':', NmServerSettings mapServerPort asString! !


!NmWorldMapLocationBroadcaster methodsFor: 'accessing' stamp: 'mu 5/29/2005 03:00'!
broadcastClient
	broadcastClient ifNil: [broadcastClient _ RbtClient new].
	^broadcastClient! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NmWorldMapLocationBroadcaster class
	instanceVariableNames: ''!

!NmWorldMapLocationBroadcaster class methodsFor: 'constants' stamp: 'mu 5/30/2005 18:52'!
defaultIntervalSeconds
	^ 12! !


NmSoapServer subclass: #NmWorldMapServer
	instanceVariableNames: 'worldMaps clearProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NetMorph-Server-MapServer'!

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 5/10/2005 17:41'!
initiClearProcess
	| clearer |
	clearProcess notNil
		ifTrue: [self release].
	clearer _ [[self clearMapsNotUsed.
			self clingTogether.
			(Delay forSeconds: self clearingIntervalSeconds) wait] repeat].
	clearProcess _ clearer forkAt: Processor userSchedulingPriority - 2! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 2/2/2005 00:59'!
initialize
	"self default initialize"
	super initialize.
	self release.
	worldMaps _ nil.
	self initiClearProcess! !

!NmWorldMapServer methodsFor: 'initializing' stamp: 'mu 2/2/2005 00:49'!
release
	clearProcess ifNotNilDo: [:p | p terminate].
	clearProcess _ nil
	! !


!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 1/21/2005 02:03'!
getMapStringNamed: mapName 
	| map |
	map _ self worldMaps
				at: mapName
				ifAbsent: [].
	^ map isNil
		ifFalse: [map csvString]! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/7/2005 10:22'!
getVersionNamed: mapName 
	| map |
	map _ self worldMaps
				at: mapName
				ifAbsent: [].
	^ map isNil
		ifFalse: [map version]! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/4/2005 02:29'!
listAllMapNames
	self clearMapsNotUsed.
	^self worldMaps keys asSortedCollection asArray
	
	! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/7/2005 10:51'!
putMapString: mapCSVString named: mapName 
	| map |
	map _ self worldMapsAt: mapName.
	map _ map fromCsvString: mapCSVString.
	map mapName: mapName.
	map incrementVersion.
	map storeNamed: mapName useServer: false.
	^ nil! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 2/2/2005 00:30'!
removeEntryWith: uuidStr named: mapName
	| map |
	map _ self worldMapsAt: mapName.
	map
		removeKey: uuidStr
		ifAbsent: [^ false].
	map storeNamed: mapName useServer: false.
	^ true! !

!NmWorldMapServer methodsFor: 'actions' stamp: 'mu 5/21/2005 18:55'!
updateEntryWith: mapEntryString named: mapName 
	| worldMap origMap newMap |
	self clearMapsNotUsed.
	worldMap _ self worldMapsAt: mapName.
	origMap _ worldMap map copy.
	worldMap
		registerDodgeIfConflict: (NmWorldMapEntry fromCsvString: mapEntryString).
	newMap _ worldMap map.
	origMap = newMap
		ifFalse: [worldMap incrementVersion.
			worldMap storeNamed: mapName useServer: false].
	^ worldMap csvString! !


!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 2/2/2005 01:03'!
clearingIntervalSeconds
	^self class defaultClearingIntervalSeconds! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 1/21/2005 01:16'!
worldMaps
	worldMaps ifNil: [worldMaps _ Dictionary new].
	^worldMaps! !

!NmWorldMapServer methodsFor: 'accessing' stamp: 'mu 1/21/2005 02:20'!
worldMapsAt: mapName 
	^ self worldMaps
		at: mapName
		ifAbsentPut: [NmWorldMap new]! !


!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 5/4/2005 18:48'!
clearMapsNotUsed
	self clearMapsNotUsedFor: self clearingIntervalSeconds.
	! !

!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 5/10/2005 18:05'!
clearMapsNotUsedFor: intervalSeconds 
	| seconds emptyMaps emptyMapKeys |
	seconds _ intervalSeconds.
	self worldMaps
		do: [:each | each removeNotUpdatedFor: seconds].
	emptyMaps _ self worldMaps
				select: [:each | each isEmpty].
	emptyMaps
		do: [:each | each forgetNamed: each mapName useServer: false].
	emptyMapKeys _ emptyMaps
				collect: [:each | each mapName].
	emptyMapKeys
		do: [:each | self worldMaps
				removeKey: each
				ifAbsent: []]! !

!NmWorldMapServer methodsFor: 'clearning' stamp: 'mu 5/10/2005 17:39'!
clingTogether
	self worldMaps
		do: [:each | 
			each clingTogetherAloneEntries].! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NmWorldMapServer class
	instanceVariableNames: ''!

!NmWorldMapServer class methodsFor: 'registering' stamp: 'mu 5/7/2005 10:23'!
registerServices
	"NmWorldMapServer registerServices"
	self registerServiceNamed: #'putMapString:named:' paramNames: #(#mapCSVString #mapName ).
	self registerServiceNamed: #'getMapStringNamed:' paramNames: #(#mapName ).
	self registerServiceNamed: #'getVersionNamed:' paramNames: #(#mapName ).
	self registerServiceNamed: #listAllMapNames paramNames: #().
	self registerServiceNamed: #'updateEntryWith:named:' paramNames: #(#mapEntryString #mapName ).
	self registerServiceNamed: #'removeEntryWith:named:' paramNames: #(#uuidStr #mapName )! !


!NmWorldMapServer class methodsFor: 'class initialization' stamp: 'mu 3/12/2003 00:19'!
initialize
	self basicInitialize! !

!NmWorldMapServer class methodsFor: 'class initialization' stamp: 'mu 2/2/2005 00:52'!
obsolete
	self release.
	super obsolete! !


!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 4/10/2005 02:08'!
exploreMaps
	"NmWorldMapServer exploreMaps"
	self default worldMaps explore! !

!NmWorldMapServer class methodsFor: 'utilities' stamp: 'mu 2/2/2005 00:51'!
release
	"self release"
	self default release! !


!NmWorldMapServer class methodsFor: 'constants' stamp: 'mu 2/2/2005 01:01'!
defaultClearingIntervalSeconds
	^20! !

NmWorldMapServer initialize!